@@ -1,4 +1,4 @@
-/*  Copyright (c) 2003-2012 Xfce Development Team
+/*  Copyright (c) 2003-2013 Xfce Development Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,19 +22,22 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <libxml/parser.h>
+#include <libsoup/soup.h>
+
+#define DATA_EXPIRY_TIME (24 * 3600)
 
 G_BEGIN_DECLS
 
-#define MAX_TIMESLICE 500
-
 enum {
     CLOUDS_PERC_LOW = 0,
-    CLOUDS_PERC_MED,
+    CLOUDS_PERC_MID,
     CLOUDS_PERC_HIGH,
     CLOUDS_PERC_CLOUDINESS,
     CLOUDS_PERC_NUM
 };
 
+typedef gpointer (*XmlParseFunc) (xmlNode *node);
+
 typedef struct {
     gchar *altitude;
     gchar *latitude;
@@ -72,8 +75,7 @@ typedef struct {
 } xml_time;
 
 typedef struct {
-    xml_time *timeslice[MAX_TIMESLICE];
-    guint num_timeslices;
+    GArray *timeslices;
     xml_time *current_conditions;
 } xml_weather;
 
@@ -99,15 +101,32 @@ typedef struct {
     gchar *longitude;
 } xml_geolocation;
 
-
 typedef struct {
     gchar *display_name;
     gchar *lat;
     gchar *lon;
 } xml_place;
 
+typedef struct {
+    gchar *altitude;
+} xml_altitude;
 
-xml_weather *parse_weather(xmlNode *cur_node);
+typedef struct {
+    gchar *country_code;
+    gchar *country_name;
+    gchar *timezone_id;
+} xml_timezone;
+
+
+xml_weather *make_weather_data(void);
+
+xml_time *make_timeslice(void);
+
+time_t parse_timestring(const gchar *ts,
+                        gchar *format);
+
+gboolean parse_weather(xmlNode *cur_node,
+                       xml_weather *wd);
 
 xml_astro *parse_astro(xmlNode *cur_node);
 
@@ -115,13 +134,27 @@ xml_geolocation *parse_geolocation(xmlNode *cur_node);
 
 xml_place *parse_place(xmlNode *cur_node);
 
-xml_time *get_timeslice(xml_weather *data,
+xml_altitude *parse_altitude(xmlNode *cur_node);
+
+xml_timezone *parse_timezone(xmlNode *cur_node);
+
+xml_time *get_timeslice(xml_weather *wd,
                         const time_t start_t,
-                        const time_t end_t);
+                        const time_t end_t,
+                        guint *index);
+
+xmlDoc *get_xml_document(SoupMessage *msg);
+
+gpointer parse_xml_document(SoupMessage *msg,
+                            XmlParseFunc parse_func);
+
+xml_time *xml_time_copy(const xml_time *src);
 
 void xml_time_free(xml_time *timeslice);
 
-void xml_weather_free(xml_weather *data);
+void xml_weather_free(xml_weather *wd);
+
+void xml_weather_clean(xml_weather *wd);
 
 void xml_astro_free(xml_astro *astro);
 
@@ -129,6 +162,10 @@ void xml_geolocation_free(xml_geolocation *geo);
 
 void xml_place_free(xml_place *place);
 
+void xml_altitude_free(xml_altitude *alt);
+
+void xml_timezone_free(xml_timezone *tz);
+
 G_END_DECLS
 
 #endif
