@@ -1,4 +1,4 @@
-/*  Copyright (c) 2003-2012 Xfce Development Team
+/*  Copyright (c) 2003-2013 Xfce Development Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,11 @@
 
 G_BEGIN_DECLS
 
-GtkWidget *create_summary_window(xfceweather_data *data);
+GtkWidget *create_summary_window(plugin_data *data);
+
+gboolean update_summary_subtitle(plugin_data *data);
+
+void summary_details_free(summary_details *sum);
 
 G_END_DECLS
 
