@@ -1,4 +1,4 @@
-/*  Copyright (c) 2003-2012 Xfce Development Team
+/*  Copyright (c) 2003-2013 Xfce Development Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,9 +34,6 @@ gchar *translate_day(gint weekday);
 
 gchar *translate_wind_direction(const gchar *wdir);
 
-gchar *translate_wind_speed(const gchar *wspeed,
-                            unit_systems unit_system);
-
 G_END_DECLS
 
 #endif
